/**
 * 分数
 */

import { FuHuaQiMyDataScoreConfig, ScoreConfig, FuHuaQiBaseDataScoreConfig, EnterpriseScoreConfig, StarConfig } from "../../../config/scoreConfig";
import { findEnterpriseList } from "../../../data/enterprise/enterprise";
import { findFuHuaQiByUSCC } from "../../../data/fuHuaQi/fuhuaqi";
import { findTaskListByParam } from "../../../data/fuHuaQi/task";

/**
 * 统计孵化器分数
 * @param uscc 孵化器统一信用代码
 */
export async function MyScore(uscc:string) { 
    let fuHuaQiInfo = await findFuHuaQiByUSCC(uscc);
    /** 初始分*/
    let startScore = fuHuaQiInfo.startScore || 400;

    /**我的数据分 */
    let isValueCount = 0;
    let valueCount = Object.keys(FuHuaQiMyDataScoreConfig).length;
    for (let key in FuHuaQiMyDataScoreConfig) {
        let type = FuHuaQiMyDataScoreConfig[key];
        if (type == '[Object]' || type == '[Number]') {
            if (fuHuaQiInfo[key] && fuHuaQiInfo[key].length) isValueCount += 1;
        } else if (fuHuaQiInfo[key]) isValueCount += 1;
    }
    let myDataScore = ScoreConfig["我的数据"] * (isValueCount / valueCount);

    /**基础数据分 */
    isValueCount = 0;
    valueCount = Object.keys(FuHuaQiBaseDataScoreConfig).length;
    for (let key in FuHuaQiBaseDataScoreConfig) {
        let type = FuHuaQiBaseDataScoreConfig[key];
        if (type == '[Object]' || type == '[Number]') {
            if (fuHuaQiInfo[key] && fuHuaQiInfo[key].length) isValueCount += 1;
        } else if (fuHuaQiInfo[key]) isValueCount += 1;
    }
    let baseDataScore = ScoreConfig["孵化器基础数据"] * (isValueCount / valueCount);

    /**我的企业数据分 */
    let myEnterpriseScore = await myEnterpriseScoreCount(uscc);

    /**任务分 */
    let taskScore = await taskScoreCount(uscc);

    let newScore = startScore + myDataScore + baseDataScore + myEnterpriseScore + taskScore;
    let oldScore = fuHuaQiInfo.socre; 
    /**根据分数算星星 */
    let oldStar = starCount(oldScore);
    let newStar = starCount(newScore);
    let starUp = newStar > oldStar;

    /**将分数写入数据 用于下一次比对 */
    fuHuaQiInfo.socre = newScore;
    await fuHuaQiInfo.save();
    
    return {scoreInfo:{star:newStar, starUp, score:newScore-oldScore}};
}


/**
 * 根据分数算星数
 * @param socre 分数
 * @returns 星数
 */
function starCount(socre:number) {
    let star = 1;
    for (let i = 0; i < StarConfig.length; i++) {
        let {name, value, starNum} = StarConfig[i];
        if ( socre <= value ) {
            star = starNum;
            break;
        }
    }
    return star;
}


/**
 * 算任务分数
 * @param uscc 孵化器统一信用代码
 * @returns 任务得分
 */
async function taskScoreCount(uscc:string) {
    let taskList = await findTaskListByParam({fuHuaQiUscc:uscc});
    let should = taskList.length;
    let completed = 0;//已完成
    let unfinished = 0;//未完成
    taskList.forEach(info => {
        if ( info.isSubmit ) completed += 1;
        else unfinished += 1;
    });
    let addScoreParam = ScoreConfig["任务分数_加分"] * 3 * 12;
    let subtractionScoreParam = ScoreConfig["任务分数_扣分"] * 3 * 12;
    return (addScoreParam * (completed/should) ) - ( subtractionScoreParam * (unfinished/should) );
}


/**
 * 我的企业分数
 * @param uscc 孵化器统一信用代码
 * @returns 我的企业得分
 */
async function myEnterpriseScoreCount(uscc:string) {
    let enterpriseList = await findEnterpriseList({fuHuaQiUscc:uscc, draftLock:true});
    let isValueCount = 0;
    let valueCount = 0;

    enterpriseList.forEach(info => {
        for (let key in EnterpriseScoreConfig) {
            let type = EnterpriseScoreConfig[key];
            if (type == '[Object]' || type == '[Number]') {
                if (info[key] && info[key].length) isValueCount += 1;
            } else if (type == 'Boolean') {
                isValueCount += 1;
            } else if (info[key]) isValueCount += 1;
        }
        valueCount += Object.keys(EnterpriseScoreConfig).length;
    });

    return ScoreConfig["企业基础数据"] * (isValueCount / valueCount);
}


// async function 
